import pickle
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt

def print_dict_keys(obj, parent_key=''):

    if isinstance(obj, dict):
        for key, value in obj.items():
            new_key = f"{parent_key}/{key}" if parent_key else key
            print(new_key)
            print_dict_keys(value, new_key)
    elif isinstance(obj, list):
        for idx, item in enumerate(obj):

            if parent_key:
                print_dict_keys(item, f"{parent_key}/idx_{idx}")
            else:
                print_dict_keys(item, f"idx_{idx}")

def get_key(data, key):

    current = data

    for idx in key.split("/"):

        if "idx" in idx:
            idx_val = int(idx.split("_")[1])

            current = current[idx_val]
        else:
            current = current[idx]

    return current

def main_log():

    path = "./data_nocam.pkl"
    reference_transform = None

    with open(path, 'rb') as fp:
        data = pickle.load(fp)
        trajectory = []

        for i in range(len(data)):

            frame = data[i]

            ego = frame["ego"]

            if reference_transform is None:

                reference_transform = frame["ego"]["world_to_ego_transform"]

            homogenous_xyz = np.append(ego["position_xyz"], 1)

            trajectory_point = (reference_transform @ homogenous_xyz)[:2]
            trajectory.append(trajectory_point)

        trajectory = np.array(trajectory)

        plt.scatter(trajectory[:, 0], trajectory[:, 1], s=2)
        plt.xlim((-2, 130))
        plt.ylim((-1, 15))


if __name__ == "__main__":
    main_log()